@@ -42,6 +42,9 @@
 #include <glib/gi18n.h>
 #include <gconf/gconf-client.h>
 
+#define GNOME_DESKTOP_USE_UNSTABLE_API
+#include <libgnome/gnome-desktop-utils.h>
+
 #include "calendar-window.h"
 
 #include "clock.h"
@@ -61,6 +64,9 @@
 #  define KEY_BIRTHDAYS_EXPANDED    "expand_birthdays"
 #  define KEY_TASKS_EXPANDED        "expand_tasks"
 #  define KEY_WEATHER_EXPANDED      "expand_weather"
+
+#  define KEY_CALENDAR_APP          "/desktop/gnome/applications/calendar"
+#  define KEY_TASKS_APP             "/desktop/gnome/applications/tasks"
 #else
 #  define N_CALENDAR_WINDOW_GCONF_PREFS 1
 #endif
@@ -137,33 +143,90 @@ static GtkWidget * create_hig_frame 		  (CalendarWindow *calwin,
 #ifdef HAVE_LIBECAL
 
 static void
-clock_launch_evolution (CalendarWindow *calwin,
-			const char     *argument)
+clock_launch_calendar_tasks_app (CalendarWindow *calwin,
+				 const char     *key_program,
+				 const char     *argument)
 {
-	char      *command_line;
-	GdkScreen *screen;
-	GError    *error;
+	char      **argv;
+	int         argc;
+	char       *key;
+	char       *program;
+	gboolean    terminal;
+	char       *command_line;
+	GdkScreen  *screen;
+	GError     *error;
+	gboolean    result;
 
-	command_line = g_find_program_in_path ("evolution");
-	if (command_line == NULL)
+	key = g_strdup_printf ("%s%s", key_program, "/exec");
+	program = gconf_client_get_string (calwin->priv->gconfclient,
+					   key, NULL);
+	g_free (key);
+
+	key = g_strdup_printf ("%s%s", key_program, "/needs_term");
+	terminal = gconf_client_get_bool (calwin->priv->gconfclient,
+					  key, NULL);
+	g_free (key);
+
+	if (program == NULL) {
+		g_printerr ("Cannot launch calendar/tasks application: key not set\n");
 		return;
+	}
+
+	command_line = g_find_program_in_path (program);
+	if (command_line == NULL) {
+		g_printerr ("Cannot launch calendar/tasks application: %s in path\n", program);
+		g_free (program);
+		return;
+	}
 	g_free (command_line);
 
-	if (argument)
-		command_line = g_strdup_printf ("evolution %s", argument);
-	else
-		command_line = g_strdup ("evolution");
+	if (argument) {
+		argc = 2;
+		argv = g_new0 (char *, 3);
+		argv[1] = g_strdup (argument);
+	} else {
+		argc = 1;
+		argv = g_new0 (char *, 2);
+	}
+	argv[0] = program; /* no strdup, and it will get freed for free */
 
 	screen = gtk_widget_get_screen (calwin->priv->calendar);
 	error = NULL;
 
-	if (!gdk_spawn_command_line_on_screen (screen, command_line,
-					       &error)) {
-		g_printerr ("Cannot launch evolution: %s\n", error->message);
+	if (terminal)
+		gnome_desktop_prepend_terminal_to_vector (&argc, &argv);
+
+	result = gdk_spawn_on_screen (screen,
+				      NULL, /* working directory */
+				      argv,
+				      NULL, /* envp */
+				      G_SPAWN_SEARCH_PATH,
+				      NULL, /* child setup func */
+				      NULL, /* user data */
+				      NULL, /* child pid */
+				      &error);
+
+	if (!result) {
+		g_printerr ("Cannot launch calendar/tasks application: %s\n",
+			    error->message);
 		g_error_free (error);
 	}
 
-	g_free (command_line);
+	g_strfreev (argv);
+}
+
+static void
+clock_launch_calendar_app (CalendarWindow *calwin,
+			   const char     *argument)
+{
+	clock_launch_calendar_tasks_app (calwin, KEY_CALENDAR_APP, argument);
+}
+
+static void
+clock_launch_tasks_app (CalendarWindow *calwin,
+			const char     *argument)
+{
+	clock_launch_calendar_tasks_app (calwin, KEY_TASKS_APP, argument);
 }
 
 static void
@@ -239,14 +302,21 @@ format_time (ClockFormat format,
             month == tm->tm_mon &&
             day   == tm->tm_mday) {
                 if (format == CLOCK_FORMAT_12)
+			/* Translators: This is a strftime format string.
+			 * It is used to display the time in 12-hours format
+			 * (eg, like in the US: 8:10 am). The %p expands to
+			 * am/pm. */
                         time_format = g_locale_from_utf8 (_("%l:%M %p"), -1, NULL, NULL, NULL);
                 else
+			/* Translators: This is a strftime format string.
+			 * It is used to display the time in 24-hours format
+			 * (eg, like in France: 20:10). */
                         time_format = g_locale_from_utf8 (_("%H:%M"), -1, NULL, NULL, NULL);
         }
         else {
-		/* Translators: If the event did not start on the current day
-		 * we will display the start date in the most abbreviated way
-		 * possible. */
+		/* Translators: This is a strftime format string.
+		 * It is used to display the start date of an appointment, in
+		 * the most abbreviated way possible. */
                 time_format = g_locale_from_utf8 (_("%b %d"), -1, NULL, NULL, NULL);
         }
 
@@ -280,12 +350,12 @@ handle_tasks_changed (CalendarWindow *calwin)
                                     TASK_COLUMN_UID,                   task->uid,
                                     TASK_COLUMN_SUMMARY,               task->summary,
                                     TASK_COLUMN_DESCRIPTION,           task->description,
-                                    TASK_COLUMN_START_TIME,            task->start_time,
-                                    TASK_COLUMN_DUE_TIME,              task->due_time,
+                                    TASK_COLUMN_START_TIME,            (gint64)task->start_time,
+                                    TASK_COLUMN_DUE_TIME,              (gint64)task->due_time,
                                     TASK_COLUMN_PERCENT_COMPLETE,      task->percent_complete,
                                     TASK_COLUMN_PERCENT_COMPLETE_TEXT, percent_complete_text,
                                     TASK_COLUMN_COMPLETED,             task->percent_complete == 100,
-                                    TASK_COLUMN_COMPLETED_TIME,        task->completed_time,
+                                    TASK_COLUMN_COMPLETED_TIME,        (gint64)task->completed_time,
                                     TASK_COLUMN_COLOR,                 task->color_string,
                                     TASK_COLUMN_PRIORITY,              task->priority,
                                     -1);
@@ -425,15 +495,19 @@ filter_out_tasks (GtkTreeModel   *model,
                   GtkTreeIter    *iter,
                   CalendarWindow *calwin)
 {
-        GTime    start_time;
-        GTime    completed_time;
-        GTime    one_day_ago;
+        gint64   start_time64;
+        gint64   completed_time64;
+        time_t   start_time;
+        time_t   completed_time;
+        time_t   one_day_ago;
         gboolean visible;
 
         gtk_tree_model_get (model, iter,
-                            TASK_COLUMN_START_TIME,     &start_time,
-                            TASK_COLUMN_COMPLETED_TIME, &completed_time,
+                            TASK_COLUMN_START_TIME,     &start_time64,
+                            TASK_COLUMN_COMPLETED_TIME, &completed_time64,
                             -1);
+        start_time = start_time64;
+        completed_time = completed_time64;
 
         one_day_ago = *(calwin->priv->current_time) - (24 * 60 * 60);
 
@@ -451,7 +525,8 @@ modify_task_text_attributes (GtkTreeModel   *model,
                              gint            column,
                              CalendarWindow *calwin)
 {
-        GTime           due_time;
+        gint64          due_time64;
+        time_t          due_time;
         PangoAttrList  *attr_list;
         PangoAttribute *attr;
         GtkTreeIter     child_iter;
@@ -469,8 +544,10 @@ modify_task_text_attributes (GtkTreeModel   *model,
         }
 
         gtk_tree_model_get (GTK_TREE_MODEL (calwin->priv->tasks_model),
-			    &child_iter, TASK_COLUMN_DUE_TIME, &due_time,
+			    &child_iter, TASK_COLUMN_DUE_TIME, &due_time64,
                             -1);
+        due_time = due_time64;
+
         if (due_time && due_time > *(calwin->priv->current_time))
                 return;
 
@@ -505,7 +582,7 @@ task_activated_cb (GtkTreeView       *view,
 
 	argument = g_strdup_printf ("task:%s", uid);
 
-	clock_launch_evolution (calwin, argument);
+	clock_launch_tasks_app (calwin, argument);
 
 	g_free (argument);
         g_free (uid);
@@ -702,12 +779,15 @@ compare_tasks  (GtkTreeModel *model,
 	else if (priority_a > priority_b)
 		return 1;
 	else {
-                GTime due_time_a, due_time_b;
+                gint64 due_time_a64, due_time_b64;
+                time_t due_time_a, due_time_b;
 
                 gtk_tree_model_get (model, a,
-				    TASK_COLUMN_DUE_TIME, &due_time_a, -1);
+				    TASK_COLUMN_DUE_TIME, &due_time_a64, -1);
                 gtk_tree_model_get (model, b,
-				    TASK_COLUMN_DUE_TIME, &due_time_b, -1);
+				    TASK_COLUMN_DUE_TIME, &due_time_b64, -1);
+                due_time_a = due_time_a64;
+                due_time_b = due_time_b64;
 
                 if (due_time_a < due_time_b)
                         return -1;
@@ -754,7 +834,7 @@ calendar_window_tree_selection_changed (GtkTreeSelection *selection,
 static void
 edit_tasks (CalendarWindow *calwin)
 {
-       clock_launch_evolution (calwin, "--component=tasks");
+	clock_launch_tasks_app (calwin, NULL);
 }
 
 static GtkWidget *
@@ -906,9 +986,9 @@ handle_appointments_changed (CalendarWindow *calwin)
                                     APPOINTMENT_COLUMN_URI,         appointment->uri,
                                     APPOINTMENT_COLUMN_SUMMARY,     appointment->summary,
                                     APPOINTMENT_COLUMN_DESCRIPTION, appointment->description,
-                                    APPOINTMENT_COLUMN_START_TIME,  appointment->start_time,
+                                    APPOINTMENT_COLUMN_START_TIME,  (gint64)appointment->start_time,
                                     APPOINTMENT_COLUMN_START_TEXT,  start_text,
-                                    APPOINTMENT_COLUMN_END_TIME,    appointment->end_time,
+                                    APPOINTMENT_COLUMN_END_TIME,    (gint64)appointment->end_time,
                                     APPOINTMENT_COLUMN_ALL_DAY,     appointment->is_all_day,
                                     APPOINTMENT_COLUMN_PIXBUF,      appointment->color_string,
                                     -1);
@@ -1020,7 +1100,7 @@ create_list_for_appointment_model (CalendarWindow      *calwin,
 static void
 edit_appointments (CalendarWindow *calwin)
 {
-       clock_launch_evolution (calwin, "--component=calendar");
+	clock_launch_calendar_app (calwin, NULL);
 }
 
 static GtkWidget *
@@ -1044,7 +1124,7 @@ create_appointment_list (CalendarWindow  *calwin,
 static void
 edit_birthdays (CalendarWindow *calwin)
 {
-       clock_launch_evolution (calwin, "--component=calendar");
+	clock_launch_calendar_app (calwin, NULL);
 }
 
 static GtkWidget *
@@ -1069,7 +1149,7 @@ create_birthday_list (CalendarWindow  *calwin,
 static void
 edit_weather (CalendarWindow *calwin)
 {
-       clock_launch_evolution (calwin, "--component=calendar");
+	clock_launch_calendar_app (calwin, NULL);
 }
 
 
@@ -1098,12 +1178,12 @@ calendar_window_create_tasks_model (CalendarWindow *calwin)
 		G_TYPE_STRING,         /* uid                     */
 		G_TYPE_STRING,         /* summary                 */
 		G_TYPE_STRING,         /* description             */
-		G_TYPE_LONG,           /* start time              */
-		G_TYPE_LONG,           /* due time                */
+		G_TYPE_INT64,          /* start time              */
+		G_TYPE_INT64,          /* due time                */
 		G_TYPE_UINT,           /* percent complete        */
 		G_TYPE_STRING,         /* percent complete text   */
 		G_TYPE_BOOLEAN,        /* completed               */
-		G_TYPE_LONG,           /* completed time          */
+		G_TYPE_INT64,          /* completed time          */
 		PANGO_TYPE_ATTR_LIST,  /* summary text attributes */
 		G_TYPE_STRING,         /* color                   */
 		G_TYPE_INT             /* priority                */
@@ -1147,9 +1227,9 @@ calendar_window_create_appointments_model (CalendarWindow *calwin)
 				    G_TYPE_STRING,   /* uri              */
 				    G_TYPE_STRING,   /* summary          */
 				    G_TYPE_STRING,   /* description      */
-				    G_TYPE_LONG,     /* start time       */
+				    G_TYPE_INT64,    /* start time       */
 				    G_TYPE_STRING,   /* start time text  */
-				    G_TYPE_LONG,     /* end time         */
+				    G_TYPE_INT64,    /* end time         */
 				    G_TYPE_BOOLEAN,  /* all day          */
 				    G_TYPE_STRING);  /* color            */
 
@@ -1192,7 +1272,7 @@ calendar_day_activated (GtkCalendar    *calendar,
 				    utc_date_tm.tm_min,
 				    0);
 
-	clock_launch_evolution (calwin, argument);
+	clock_launch_calendar_app (calwin, argument);
 
 	g_free (argument);
 }
