@@ -25,7 +25,7 @@
 #ifndef __PANEL_COMPATIBILITY_H__
 #define __PANEL_COMPATIBILITY_H__
 
-#include <glib/gmacros.h>
+#include <glib.h>
 #include <gconf/gconf-client.h>
 
 G_BEGIN_DECLS
