@@ -50,6 +50,7 @@ struct _PanelDItemEditorPrivate
 	/* the revert ditem will only contain relevant keys */
 	GKeyFile *revert_key_file;
 
+	gboolean  reverting;
 	gboolean  dirty;
 	guint     save_timeout;
 
@@ -141,10 +142,12 @@ static RevertKey revert_keys [] = {
 	{ "Icon",     G_TYPE_STRING,  FALSE, TRUE  },
 	{ "Name",     G_TYPE_STRING,  FALSE, TRUE  },
 	{ "Comment",  G_TYPE_STRING,  FALSE, TRUE  },
+	{ "X-GNOME-FullName", G_TYPE_STRING,  FALSE, TRUE  },
 	/* C version of those keys */
 	{ "Icon",     G_TYPE_STRING,  FALSE, FALSE },
 	{ "Name",     G_TYPE_STRING,  FALSE, FALSE },
-	{ "Comment",  G_TYPE_STRING,  FALSE, FALSE }
+	{ "Comment",  G_TYPE_STRING,  FALSE, FALSE },
+	{ "X-GNOME-FullName", G_TYPE_STRING,  FALSE, FALSE }
 };
 
 enum {
@@ -858,12 +861,21 @@ panel_ditem_editor_name_changed (PanelDItemEditor *dialog)
 
 	name = gtk_entry_get_text (GTK_ENTRY (dialog->priv->name_entry));
 
-	if (name && name[0])
-		panel_key_file_set_locale_string (dialog->priv->key_file,
-						  "Name", name);
-	else
+	if (!dialog->priv->reverting) {
+		/* When reverting, we don't need to set the content of the key
+		 * file; we only want to send a signal. Changing the key file
+		 * could actually break the revert since it might overwrite the
+		 * old Name value with the X-GNOME-FullName value */
+		if (name && name[0])
+			panel_key_file_set_locale_string (dialog->priv->key_file,
+							  "Name", name);
+		else
+			panel_key_file_remove_all_locale_key (dialog->priv->key_file,
+							      "Name");
+
 		panel_key_file_remove_all_locale_key (dialog->priv->key_file,
-						      "Name");
+						      "X-GNOME-FullName");
+	}
 
 	g_signal_emit (G_OBJECT (dialog), ditem_edit_signals[NAME_CHANGED], 0,
 		       name);
@@ -1113,6 +1125,48 @@ panel_ditem_editor_connect_signals (PanelDItemEditor *dialog)
 	g_signal_connect (dialog, "response", G_CALLBACK (response_cb), NULL);
 }
 
+static void
+panel_ditem_editor_block_signals (PanelDItemEditor *dialog)
+{
+	PanelDItemEditorPrivate *priv;
+
+	priv = dialog->priv;
+
+#define BLOCK_CHANGED(widget, callback) \
+	g_signal_handlers_block_by_func (G_OBJECT (widget), \
+					 G_CALLBACK (callback), \
+					 dialog); \
+	g_signal_handlers_block_by_func (G_OBJECT (widget), \
+					 G_CALLBACK (panel_ditem_editor_changed), \
+					 dialog);
+	BLOCK_CHANGED (priv->type_combo, type_combo_changed);
+	BLOCK_CHANGED (priv->name_entry, panel_ditem_editor_name_changed);
+	BLOCK_CHANGED (priv->command_entry, panel_ditem_editor_command_changed);
+	BLOCK_CHANGED (priv->comment_entry, panel_ditem_editor_comment_changed);
+	BLOCK_CHANGED (priv->icon_entry, panel_ditem_editor_icon_changed);
+}
+
+static void
+panel_ditem_editor_unblock_signals (PanelDItemEditor *dialog)
+{
+	PanelDItemEditorPrivate *priv;
+
+	priv = dialog->priv;
+
+#define UNBLOCK_CHANGED(widget, callback) \
+	g_signal_handlers_unblock_by_func (G_OBJECT (widget), \
+					   G_CALLBACK (callback), \
+					   dialog); \
+	g_signal_handlers_unblock_by_func (G_OBJECT (widget), \
+					   G_CALLBACK (panel_ditem_editor_changed), \
+					   dialog);
+	UNBLOCK_CHANGED (priv->type_combo, type_combo_changed);
+	UNBLOCK_CHANGED (priv->name_entry, panel_ditem_editor_name_changed);
+	UNBLOCK_CHANGED (priv->command_entry, panel_ditem_editor_command_changed);
+	UNBLOCK_CHANGED (priv->comment_entry, panel_ditem_editor_comment_changed);
+	UNBLOCK_CHANGED (priv->icon_entry, panel_ditem_editor_icon_changed);
+}
+
 static void
 panel_ditem_editor_init (PanelDItemEditor *dialog)
 {
@@ -1127,6 +1181,7 @@ panel_ditem_editor_init (PanelDItemEditor *dialog)
 	priv->key_file = NULL;
 	priv->free_key_file = FALSE;
 	priv->revert_key_file = NULL;
+	priv->reverting = FALSE;
 	priv->dirty = FALSE;
 	priv->save_timeout = 0;
 	priv->uri = NULL;
@@ -1159,7 +1214,7 @@ type_combo_changed (PanelDItemEditor *dialog)
 		}
 		break;
 	case PANEL_DITEM_EDITOR_TYPE_TERMINAL_APPLICATION:
-		text = _("_Command:");
+		text = _("Comm_and:");
 		if (dialog->priv->combo_setuped) {
 			panel_key_file_set_string (dialog->priv->key_file,
 						   "Type", "Application");
@@ -1256,7 +1311,9 @@ panel_ditem_editor_sync_display (PanelDItemEditor *dialog)
 	key_file = dialog->priv->key_file;
 
 	/* Name */
-	buffer = panel_key_file_get_locale_string (key_file, "Name");
+	buffer = panel_key_file_get_locale_string (key_file, "X-GNOME-FullName");
+	if (!buffer)
+		buffer = panel_key_file_get_locale_string (key_file, "Name");
 	gtk_entry_set_text (GTK_ENTRY (dialog->priv->name_entry),
 			    buffer ? buffer : "");
 	g_free (buffer);
@@ -1490,6 +1547,8 @@ panel_ditem_editor_revert (PanelDItemEditor *dialog)
 
 	g_return_if_fail (PANEL_IS_DITEM_EDITOR (dialog));
 
+	dialog->priv->reverting = TRUE;
+
 	key_file = dialog->priv->key_file;
 	revert_key_file = dialog->priv->revert_key_file;
 
@@ -1549,6 +1608,8 @@ panel_ditem_editor_revert (PanelDItemEditor *dialog)
 							panel_ditem_editor_save_timeout,
 							dialog);
 	}
+
+	dialog->priv->reverting = FALSE;
 }
 
 static void
@@ -1607,7 +1668,11 @@ panel_ditem_editor_set_revert (PanelDItemEditor *dialog)
 static void
 panel_ditem_editor_key_file_loaded (PanelDItemEditor  *dialog)
 {
+	/* the user is not changing any value here, so block the signals about
+	 * changing a value */
+	panel_ditem_editor_block_signals (dialog);
 	panel_ditem_editor_sync_display (dialog);
+	panel_ditem_editor_unblock_signals (dialog);
 
 	/* This should be after panel_ditem_editor_sync_display ()
 	 * so the revert button is insensitive */
