@@ -25,7 +25,7 @@
 #ifndef __PANEL_ENUMS_H__
 #define __PANEL_ENUMS_H__
 
-#include <glib/gmacros.h>
+#include <glib.h>
 
 G_BEGIN_DECLS
 
@@ -95,7 +95,7 @@ typedef enum {
         PANEL_ACTION_SEARCH,
 	PANEL_ACTION_FORCE_QUIT,
 	PANEL_ACTION_CONNECT_SERVER,
-        PANEL_ACTION_SHUTDOWN,
+	PANEL_ACTION_SHUTDOWN,
 	/* compatibility with GNOME < 2.13.90 */
         PANEL_ACTION_SCREENSHOT,
         PANEL_ACTION_LAST
