@@ -29,13 +29,12 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+
 #include <cairo.h>
-#include <cairo-xlib.h> //FIXME should be removed when gdk_cairo_set_source_pixmap() is available (GTK+ 2.10)
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtktooltips.h>
+#include <gtk/gtk.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-types.h>
@@ -43,7 +42,6 @@
 #include <bonobo/bonobo-item-handler.h>
 #include <bonobo/bonobo-shlib-factory.h>
 #include <bonobo/bonobo-property-bag-client.h>
-#include <libgnome/gnome-program.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <X11/Xatom.h>
@@ -127,29 +125,57 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 
 	list = gconf_client_all_entries (client, schema_dir, error);
 
-	g_return_if_fail (*error == NULL);
+	if (*error != NULL)
+		return;
 
 	for (l = list; l; l = l->next) {
-		GConfEntry *entry = l->data;
-		gchar      *key;
-		gchar      *tmp;
+		GConfEntry  *entry = l->data;
+		const gchar *schema_key;
+		GConfEntry  *applet_entry;
+		const gchar *applet_schema_key;
+		gchar       *key;
+		gchar       *tmp;
 
-		tmp = g_path_get_basename (gconf_entry_get_key (entry));
+		schema_key = gconf_entry_get_key (entry);
+		tmp = g_path_get_basename (schema_key);
 
 		if (strchr (tmp, '-'))
 			g_warning ("Applet key '%s' contains a hyphen. Please "
 				   "use underscores in gconf keys\n", tmp);
 
 		key = g_strdup_printf ("%s/%s", prefs_key, tmp);
-
 		g_free (tmp);
 
-		gconf_engine_associate_schema (
-			client->engine, key, gconf_entry_get_key (entry), error);
+		/* Associating a schema is potentially expensive, so let's try
+		 * to avoid this by doing it only when needed. So we check if
+		 * the key is already correctly associated. */
+
+		applet_entry = gconf_client_get_entry (client, key,
+						       NULL, TRUE, NULL);
+		if (applet_entry)
+			applet_schema_key = gconf_entry_get_schema_name (applet_entry);
+		else
+			applet_schema_key = NULL;
+
+		if (g_strcmp0 (schema_key, applet_schema_key) != 0) {
+			gconf_engine_associate_schema (client->engine,
+						       key, schema_key, error);
+
+			if (applet_entry == NULL ||
+			    gconf_entry_get_value (applet_entry) == NULL ||
+			    gconf_entry_get_is_default (applet_entry)) {
+				/* unset the key: gconf_client_get_entry()
+				 * brought an invalid entry in the client
+				 * cache, and we want to fix this */
+				gconf_client_unset (client, key, NULL);
+			}
+		}
 
 		g_free (key);
 
-		gconf_entry_free (entry);
+		if (applet_entry)
+			gconf_entry_unref (applet_entry);
+		gconf_entry_unref (entry);
 
 		if (*error) {
 			g_slist_free (list);
@@ -172,8 +198,10 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 		prefs_subdir  = g_strdup_printf ("%s/%s", prefs_key, tmp);
 		schema_subdir = g_strdup_printf ("%s/%s", schema_dir, tmp);
 
-		panel_applet_associate_schemas_in_dir (
-			client, prefs_subdir, schema_subdir, error);
+		panel_applet_associate_schemas_in_dir (client,
+						       prefs_subdir,
+						       schema_subdir,
+						       error);
 
 		g_free (prefs_subdir);
 		g_free (schema_subdir);
@@ -208,8 +236,10 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	else
 		error = &our_error;
 
-	panel_applet_associate_schemas_in_dir (
-		applet->priv->client, applet->priv->prefs_key, schema_dir, error);
+	panel_applet_associate_schemas_in_dir (applet->priv->client,
+					       applet->priv->prefs_key,
+					       schema_dir,
+					       error);
 
 	if (!opt_error && our_error) {
 		g_warning (G_STRLOC ": failed to add preferences from '%s' : '%s'",
@@ -663,7 +693,7 @@ panel_applet_can_focus (GtkWidget *widget)
 	 * A PanelApplet widget can focus if it has a tooltip or it does 
 	 * not have any focusable children.
 	 */
-	if (gtk_tooltips_data_get (widget))
+	if (gtk_widget_get_has_tooltip (widget))
 		return TRUE;
 
 	if (!PANEL_IS_APPLET (widget))
@@ -860,8 +890,8 @@ panel_applet_focus (GtkWidget        *widget,
 	}
 
 	previous_focus_child = GTK_CONTAINER (widget)->focus_child;
-	 if (!previous_focus_child && !GTK_WIDGET_HAS_FOCUS (widget)) {
-		if (gtk_tooltips_data_get (widget)) {
+	if (!previous_focus_child && !GTK_WIDGET_HAS_FOCUS (widget)) {
+		if (gtk_widget_get_has_tooltip (widget)) {
 			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
 			gtk_widget_grab_focus (widget);
 			GTK_WIDGET_UNSET_FLAGS (widget, GTK_CAN_FOCUS);
@@ -869,11 +899,12 @@ panel_applet_focus (GtkWidget        *widget,
 		}
 	}
 	ret = GTK_WIDGET_CLASS (panel_applet_parent_class)->focus (widget, dir);
+
 	if (!ret && !previous_focus_child) {
  		if (!GTK_WIDGET_HAS_FOCUS (widget))  {
 			/*
 			 * Applet does not have a widget which can focus so set
-			 * the focus on the applet unless it aleready had focus
+			 * the focus on the applet unless it already had focus
 			 * because it had a tooltip.
 			 */ 
 			GTK_WIDGET_SET_FLAGS (widget, GTK_CAN_FOCUS);
@@ -961,9 +992,7 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 	int              width;
 	int              height;
 	cairo_t         *cr;
-	cairo_surface_t *surface;
 	cairo_pattern_t *pattern;
-	cairo_matrix_t   matrix;
 
 	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
 
@@ -990,32 +1019,23 @@ panel_applet_get_pixmap (PanelApplet     *applet,
 		return NULL;
 	}
 
-	gdk_drawable_get_size (GDK_DRAWABLE (pixmap), &width, &height);
-
-	//FIXME gdk_cairo_set_source_pixmap() should do the trick (GTK+ 2.10)
-	surface = cairo_xlib_surface_create (GDK_DISPLAY_XDISPLAY (display),
- 					     //xid,
-					     gdk_x11_drawable_get_xid (GDK_DRAWABLE (pixmap)),
-					     gdk_x11_visual_get_xvisual (gdk_drawable_get_visual (GTK_WIDGET (applet)->window)),
-					     width, height);
-
-	pattern = cairo_pattern_create_for_surface (surface);
-	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
-	cairo_matrix_init_translate (&matrix, x, y);
-	cairo_pattern_set_matrix (pattern, &matrix);
-
 	gdk_drawable_get_size (GDK_DRAWABLE (GTK_WIDGET (applet)->window),
 			       &width, &height);
 	retval = gdk_pixmap_new (GTK_WIDGET (applet)->window,
 				 width, height, -1);
 
+	/* the pixmap has no colormap, and we need one */
+	gdk_drawable_set_colormap (GDK_DRAWABLE (pixmap),
+				   gdk_drawable_get_colormap (GTK_WIDGET (applet)->window));
+
 	cr = gdk_cairo_create (GDK_DRAWABLE (retval));
-	cairo_set_source (cr, pattern);
+	gdk_cairo_set_source_pixmap (cr, pixmap, -x, -y);
+	pattern = cairo_get_source (cr);
+	cairo_pattern_set_extend (pattern, CAIRO_EXTEND_REPEAT);
+
 	cairo_rectangle (cr, 0, 0, width, height);
 	cairo_fill (cr);
 
-	cairo_pattern_destroy (pattern);
-	cairo_surface_destroy (surface);
 	cairo_destroy (cr);
 
 	g_object_unref (pixmap);
@@ -1049,7 +1069,7 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 
 		if (!elements [1] || !panel_applet_parse_color (elements [1], color)) {
 
-			g_warning (_("Incomplete '%s' background type received"), elements [0]);
+			g_warning ("Incomplete '%s' background type received", elements [0]);
 			g_strfreev (elements);
 			return PANEL_NO_BACKGROUND;
 		}
@@ -1063,7 +1083,7 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 		g_return_val_if_fail (pixmap != NULL, PANEL_NO_BACKGROUND);
 
 		if (!panel_applet_parse_pixmap_str (elements [1], &pixmap_id, &x, &y)) {
-			g_warning (_("Incomplete '%s' background type received: %s"),
+			g_warning ("Incomplete '%s' background type received: %s",
 				   elements [0], elements [1]);
 
 			g_strfreev (elements);
@@ -1072,14 +1092,14 @@ panel_applet_handle_background_string (PanelApplet  *applet,
 
 		*pixmap = panel_applet_get_pixmap (applet, pixmap_id, x, y);
 		if (!*pixmap) {
-			g_warning (_("Failed to get pixmap %s"), elements [1]);
+			g_warning ("Failed to get pixmap %s", elements [1]);
 			g_strfreev (elements);
 			return PANEL_NO_BACKGROUND;
 		}
 
 		retval = PANEL_PIXMAP_BACKGROUND;
 	} else
-		g_warning (_("Unknown background type received"));
+		g_warning ("Unknown background type received");
 
 	g_strfreev (elements);
 
@@ -1160,7 +1180,7 @@ panel_applet_update_background_for_widget (GtkWidget                 *widget,
 	gtk_widget_set_style (widget, NULL);
 	rc_style = gtk_rc_style_new ();
 	gtk_widget_modify_style (widget, rc_style);
-	gtk_rc_style_unref (rc_style);
+	g_object_unref (rc_style);
 
 	switch (type) {
 	case PANEL_NO_BACKGROUND:
@@ -1303,7 +1323,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_ORIENT_IDX,
 				 BONOBO_ARG_SHORT,
 				 NULL,
-				 _("The Applet's containing Panel's orientation"),
+				 "The Applet's containing Panel's orientation",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1311,7 +1331,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_SIZE_IDX,
 				 BONOBO_ARG_SHORT,
 				 NULL,
-				 _("The Applet's containing Panel's size in pixels"),
+				 "The Applet's containing Panel's size in pixels",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1319,7 +1339,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_BACKGROUND_IDX,
 				 BONOBO_ARG_STRING,
 				 NULL,
-				 _("The Applet's containing Panel's background color or pixmap"),
+				 "The Applet's containing Panel's background color or pixmap",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1327,7 +1347,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_FLAGS_IDX,
 				 BONOBO_ARG_SHORT,
 				 NULL,
-				 _("The Applet's flags"),
+				 "The Applet's flags",
 				 Bonobo_PROPERTY_READABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1335,7 +1355,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_SIZE_HINTS_IDX,
 				 TC_CORBA_sequence_CORBA_long,
 				 NULL,
-				 _("Ranges that hint what sizes are acceptable for the applet"),
+				 "Ranges that hint what sizes are acceptable for the applet",
 				 Bonobo_PROPERTY_READABLE);
 
 	bonobo_property_bag_add (sack,
@@ -1343,7 +1363,7 @@ panel_applet_property_bag (PanelApplet *applet)
 				 PROPERTY_LOCKED_DOWN_IDX,
 				 BONOBO_ARG_BOOLEAN,
 				 NULL,
-				 _("The Applet's containing Panel is locked down"),
+				 "The Applet's containing Panel is locked down",
 				 Bonobo_PROPERTY_READABLE | Bonobo_PROPERTY_WRITEABLE);
 
 	return sack;
